--[[--------------------------------------------------------------------------------
MACIULXOCHITL                                                                      -
files.lua                                                                          -
Global values and IO operations                                                    -
]]----------------------------------------------------------------------------------

def = {
  --   First goes defaults / not saving /
  SAMPLE_MAX_AMPLITUDE = 1.0,
  SAMPLE_MIN_AMPLITUDE = -1.0,
  SAMPLE_MAX_PHASE = 1024,
  SAMPLE_FREQUENCY = 44100,
  SAMPLE_BIT_DEPTH = 16,
  SAMPLE_CHANNELS = 1,
  SAMPLE_MAX_NOISE = 10,
  SAMPLE_MAX_PERIOD = 64,
  GAUSSIAN_V_MAX = 1.0,
  GAUSSIAN_V_MIN = 0.0,
  GAUSSIAN_N_MAX = 10.0,
  GAUSSIAN_N_MIN = 0.0,
  GAUSSIAN_S_MAX = 0.0,
  GAUSSIAN_S_MIN = -10.0,
}

tmp = {
  -- now get saved values
  INSTRUMENT_NAME = "default",

  SAMPLE_NAME = "Sample",
  SAMPLE_FRAMES = 84,
  SAMPLE_POPUP = 1,
  SAMPLE_FREQUENCY_1 = 1.0,
  SAMPLE_FREQUENCY_2 = 1.0,
  SAMPLE_FREQUENCY_3 = 1.0,
  SAMPLE_AMPLITUDE_1 = 0.5,
  SAMPLE_AMPLITUDE_2 = 0.5,
  SAMPLE_AMPLITUDE_3 = 0.5,
  SAMPLE_PHASE_1 = 0,
  SAMPLE_PHASE_2 = 0,
  SAMPLE_PHASE_3 = 0,
  SAMPLE_TYPE_1 = 1,
  SAMPLE_TYPE_2 = 1,
  SAMPLE_TYPE_3 = 1,
  SAMPLE_DETAIL_1 = 1,
  SAMPLE_DETAIL_2 = 1,
  SAMPLE_DETAIL_3 = 1,
  SAMPLE_BEAT_SYNC_ENABLED = false,
  SAMPLE_BEAT_SYNC_MODE = 1,
  SAMPLE_BEAT_SYNC_LINES = 16,
  SAMPLE_LOOP_MODE = 2,
  SAMPLE_LOOP_RELEASE = false,
  SAMPLE_LOOP_START = 1,
  SAMPLE_LOOP_END = 512,
  SAMPLE_PANNING = 0.5,
  SAMPLE_VOLUME = 1.0,
  SAMPLE_TRANSPOSE = 0.0,
  SAMPLE_FINE_TUNE = 0.0,
  SAMPLE_INTERPOLATION_MODE = 1,
  SAMPLE_OVERSAMPLE_ENABLED = false,
  SAMPLE_NEW_NOTE_ACTION = 1,
  SAMPLE_ONESHOT = false,
  SAMPLE_MUTE_GROUP = 0,
  SAMPLE_AUTOSEEK = false,
  SAMPLE_AUTOFADE = false,

  SAMPLE_NOISE_BOOL = false,
  SAMPLE_NOISE_AMOUNT = 0.0,

  
  GAUSSIAN_ON_OFF = false,
  GAUSSIAN_V1 = 0.1,
  GAUSSIAN_V2 = 0.2,
  GAUSSIAN_V3 = 0.3,
  GAUSSIAN_V4 = 0.4,
  GAUSSIAN_V5 = 0.5,

  GAUSSIAN_N1 = 1,
  GAUSSIAN_N2 = 2.2,
  GAUSSIAN_N3 = 3,
  GAUSSIAN_N4 = 4.2,
  GAUSSIAN_N5 = 4.8,

  GAUSSIAN_S1 = -1,
  GAUSSIAN_S2 = -2,
  GAUSSIAN_S3 = -3,
  GAUSSIAN_S4 = -4,
  GAUSSIAN_S5 = -5,

  OP1 = 1,
  OP2 = 1,

  OSC1_FREQUENCY_MOD = false,
  OSC1_AMPLITUDE_MOD = false,
  OSC1_PHASE_MOD = false,

  OSC2_FREQUENCY_MOD = false,
  OSC2_AMPLITUDE_MOD = false,
  OSC2_PHASE_MOD = false,

  OSC3_FREQUENCY_MOD = false,
  OSC3_AMPLITUDE_MOD = false,
  OSC3_PHASE_MOD = false,
}

-- how to deepcopy this !!!
-- default = {unpack(tmp)}
default = {}
for key, value in pairs(tmp) do
  default[key] = value
end


--[[-------------------------------------------------------------
Load config file as Lua 
]]---------------------------------------------------------------
function load_config(filename)
  
  -- load lua file and parse it
  local ftables,err = loadfile(filename)
  
  if err then
    renoise.app():show_error("ERROR:Can't load patch file" .. filename)
    return
  end
  
  local tables = ftables()
  
  for idx = 1,#tables do
    
    local tolinki = {}
    for i,v in pairs(tables[idx]) do
        
        if type(v) == "table" then
          tables[idx][i] = tables[v[1]]
        end
        
        if type(i) == "table" and tables[i[1]] then
          table.insert(tolinki, {i,tables[i[1]]})
        end
    
    end
    
    -- link indices
    for _,v in ipairs(tolinki) do
      tables[idx][v[2]],tables[idx][v[1]] = tables[idx][v[1]],nil
    end
  
  end
  return tables[1]
end


--[[-------------------------------------------------------------
Load patch file                                                 -
]]---------------------------------------------------------------
function load_patch_file(filename)

  if filename then
    tmp = load_config(filename)

    if not tmp then
      renoise.app():show_warning("This isn't Macuilxochitl config file !!!")
      tmp = default
    end
  else
    tmp = default
  end

  -- refresh gui widgets
 
  vb.views.freq1slider.value = tmp.SAMPLE_FREQUENCY_1
  vb.views.amp1slider.value = tmp.SAMPLE_AMPLITUDE_1
  vb.views.phase1slider.value = tmp.SAMPLE_PHASE_1
  vb.views.detail1slider.value = tmp.SAMPLE_DETAIL_1
  vb.views.osc1type.value = tmp.SAMPLE_TYPE_1
  
  if tmp.SAMPLE_TYPE_1 < 5 then 
    vb.views.osc1detail.visible = false
  else
    vb.views.osc1detail.visible = true
  end

  vb.views.freq2slider.value = tmp.SAMPLE_FREQUENCY_2
  vb.views.amp2slider.value = tmp.SAMPLE_AMPLITUDE_2
  vb.views.phase2slider.value = tmp.SAMPLE_PHASE_2
  vb.views.detail2slider.value = tmp.SAMPLE_DETAIL_2
  vb.views.osc2type.value = tmp.SAMPLE_TYPE_2
  
  if tmp.SAMPLE_TYPE_2 < 5 then 
    vb.views.osc2detail.visible = false
  else
    vb.views.osc2detail.visible = true
  end
  
  vb.views.freq3slider.value = tmp.SAMPLE_FREQUENCY_3
  vb.views.amp3slider.value = tmp.SAMPLE_AMPLITUDE_3
  vb.views.phase3slider.value = tmp.SAMPLE_PHASE_3
  vb.views.detail3slider.value = tmp.SAMPLE_DETAIL_3
  vb.views.osc3type.value = tmp.SAMPLE_TYPE_3
  
  if tmp.SAMPLE_TYPE_3 < 5 then 
    vb.views.osc3detail.visible = false
  else
    vb.views.osc3detail.visible = true
  end

  vb.views.osc2freqmod.value = tmp.OSC2_FREQUENCY_MOD
  vb.views.osc2ampmod.value = tmp.OSC2_AMPLITUDE_MOD
  vb.views.osc2phasemod.value = tmp.OSC2_PHASE_MOD
  vb.views.osc3freqmod.value = tmp.OSC3_FREQUENCY_MOD
  vb.views.osc3ampmod.value = tmp.OSC3_AMPLITUDE_MOD
  vb.views.osc3phasemod.value = tmp.OSC3_PHASE_MOD

  vb.views.noise.value = tmp.SAMPLE_NOISE_AMOUNT
  
  if tmp.SAMPLE_NOISE_AMOUNT < 1 then
    vb.views.noisy_message.text = "Sak Nik sleeps"
  else
    vb.views.noisy_message.text = "Sak Nik is awake"
  end

  vb.views.framepopup.value = tmp.SAMPLE_POPUP
  vb.views.framebox.value = tmp.SAMPLE_FRAMES
  tmp.SAMPLE_FRAMES = tmp.SAMPLE_FRAMES

  vb.views.op1.value = tmp.OP1
  vb.views.op2.value = tmp.OP2

  if tmp.GAUSSIAN_ON_OFF then
    vb.views.gaussian_on_off.value = true
  else
    vb.views.gaussian_on_off.value = false
  end

  vb.views.gauss_v1.value = tmp.GAUSSIAN_V1
  vb.views.gauss_v2.value = tmp.GAUSSIAN_V2
  vb.views.gauss_v3.value = tmp.GAUSSIAN_V3
  vb.views.gauss_v4.value = tmp.GAUSSIAN_V4
  vb.views.gauss_v5.value = tmp.GAUSSIAN_V5
  vb.views.gauss_n1.value = tmp.GAUSSIAN_N1
  vb.views.gauss_n2.value = tmp.GAUSSIAN_N2
  vb.views.gauss_n3.value = tmp.GAUSSIAN_N3
  vb.views.gauss_n4.value = tmp.GAUSSIAN_N4
  vb.views.gauss_n5.value = tmp.GAUSSIAN_N5
  vb.views.gauss_s1.value = tmp.GAUSSIAN_S1
  vb.views.gauss_s2.value = tmp.GAUSSIAN_S2
  vb.views.gauss_s3.value = tmp.GAUSSIAN_S3
  vb.views.gauss_s4.value = tmp.GAUSSIAN_S4
  vb.views.gauss_s5.value = tmp.GAUSSIAN_S5
 
end

--[[-------------------------------------------------------------
Save config file as Lua file
]]---------------------------------------------------------------
function save_config(filename, tab)
  local file, error = io.open(filename, "wb")
  
  if error then
    renoise.app():show_error("Can't save file " .. config_table)
    return
  end

  --table.sort(tab)
  
  file:write("return {\n-- Macuilxochitl Config file\n{\n")
  
  for key, value in pairs(tab) do
    if type(value) == "boolean" then
      if value then value="true" else value="false" end
    elseif type(value) == "string" then
      value = '"' .. tostring(value) .. '"'
      --value = string.format("%q", value)
    end
    file:write('\t["' .. key .. '"] = ' .. value .. ",\n")
  end
  
  file:write("},\n}")
  file:close()

end

--[[-------------------------------------------------------------
Save patch                                                      -
]]---------------------------------------------------------------
function save_patch_file(filename)

  local instrument = renoise.song().selected_instrument
  local sample = instrument:sample(1)
  tmp.INSTRUMENT_NAME = tostring(instrument.name)
  if sample.name then
    tmp.SAMPLE_NAME = tostring(sample.name)
  else
    tmp.SAMPLE_NAME = tostring(filename)
  end
  read_info_from_sample(sample, instrument)
  save_config(filename, tmp)

end
