--[[----------------------------------------------------------------
Macuilxochitl GUI file                                             -
]]------------------------------------------------------------------

--require "src/generators"


OPERATIONS = {"Nope","Add", "Sub", "Mul", "Div", "Min", "Max", "Mod"}
WAVES = {"Simple Saw", "Simple Sine", "Simple Square", "Simple Triangle",
         "Sin to Saw", "Sin to Square", "Sin to Tan"}
BUFFERS = {"84", "169", "337", "674", "1348", "2697", "5395", "10790"}
NOTES = {"C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B"}

--[[----------------------------------------------------------------
gui constants                                                      -
]]------------------------------------------------------------------

DEFAULT_MARGIN = renoise.ViewBuilder.DEFAULT_CONTROL_MARGIN
DEFAULT_SPACING = renoise.ViewBuilder.DEFAULT_CONTROL_SPACING
DEFAULT_HEIGHT = renoise.ViewBuilder.DEFAULT_CONTROL_HEIGHT
MINISLIDER_HEIGHT = 12
SELECTED_PAGE = 3
MENU_BUTTON_WIDTH = 32
MENU_BUTTON_HEIGHT= 24
OSC_BUTTON_WH = 19
VALUEBOX_HEIGHT = 20


--[[----------------------------------------------------------------
Show GUI                                                           -
]]------------------------------------------------------------------


function show_gui()

--[[----------------------------------------------------------------
Create OSC 1 settings                                              -
]]------------------------------------------------------------------
  local osc_1_gui = vb:column{
    id="osc1panel",
    style="group",
    margin=DEFAULT_MARGIN,
    spacing=DEFAULT_SPACING,
    height = DEFAULT_HEIGHT,
    vb:row{
      vb:text{
        text="Huehuecóyotl",
        font="bold",
        style="strong",
        width=90,
      },
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:popup{
          id="osc1type",
          midi_mapping="MX:OSC1_selector",
          width=140,
          items=WAVES,
          value=tmp.SAMPLE_TYPE_1,
          notifier = function(wave)
            if wave < 5 then
              vb.views.osc1detail.visible = false
            else
              vb.views.osc1detail.visible = true
            end
            tmp.SAMPLE_TYPE_1 = wave
            OSC1.class = tmp.SAMPLE_TYPE_1
            redraw_sample()
          end
        },
      },
    },
    -- --------------------------------------- frequency OSC 1
    vb:row{
      vb:text{text="Period", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:slider{
          --height=VALUEBOX_HEIGHT,
          id = "freq1slider",
          midi_mapping="MX:OSC1_period",
          width=140,
          min=1,
          max=64,
          value=tmp.SAMPLE_FREQUENCY_1,
          notifier = function(freq)
            tmp.SAMPLE_FREQUENCY_1 = freq
            OSC1.frequency = tmp.SAMPLE_FREQUENCY_1
            redraw_sample()
          end
        },
      },
    },
    -- --------------------------------------- amplitude OSC 1
    vb:row{
      vb:text{text="Amplitude", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:slider{
          id = "amp1slider",
          midi_mapping="MX:OSC1_amplitude",
          width=140,
          min=def.SAMPLE_MIN_AMPLITUDE,
          max=def.SAMPLE_MAX_AMPLITUDE,
          value=tmp.SAMPLE_AMPLITUDE_1,
          notifier = function(amplitude)
            tmp.SAMPLE_AMPLITUDE_1 = amplitude
            OSC1.amplitude = tmp.SAMPLE_AMPLITUDE_1
            redraw_sample()
          end
        },
      },
    },
    -- ----------------------------------------- phase OSC 1
    vb:row{
      vb:text{text="Phase", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:valuebox{
          id = "phase1slider",
          midi_mapping="MX:OSC1_phase",
          width=140,
          min=-def.SAMPLE_MAX_PHASE,
          max=def.SAMPLE_MAX_PHASE,
          value=tmp.SAMPLE_PHASE_1,
          steps={1, 10},
          notifier = function(phase)
            tmp.SAMPLE_PHASE_1 = phase
            OSC1.phase = tmp.SAMPLE_PHASE_1
            redraw_sample()
          end
        },
      },
    },
    ------------------------------------------------ Detail OSC1
    vb:row{
      id="osc1detail",
      vb:text{text="Detail", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:valuebox{
          id = "detail1slider",
          midi_mapping="MX:OSC1_detail",
          width=140,
          min=1,
          max=128,
          value=tmp.SAMPLE_DETAIL_1,
          steps={1, 10},
          notifier = function(detail)
            tmp.SAMPLE_DETAIL_1 = math.floor(detail)
            OSC1.detail = tmp.SAMPLE_DETAIL_1
            redraw_sample()
          end
        },
      },
    },
  }

--[[--------------------------------------------------------------
Oscilator 2 Settings                                             -
]]----------------------------------------------------------------
  local osc_2_gui = vb:column{
    id="osc2panel",
    style="group",
    margin=DEFAULT_MARGIN,
    vb:horizontal_aligner{
      mode="left",
      vb:text{
        text="Ah-Xoc-Xin",
        font="bold",
        style="strong",
        width=90,
      },
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:horizontal_aligner{
          mode="right",
          width=160,
          spacing=5,
          vb:popup{
            id="osc2type",
            midi_mapping="MX:OSC2_selector",
            width=140,
            items=WAVES,
            value=tmp.SAMPLE_TYPE_2,
            notifier = function(wave)
              if wave < 5 then
                vb.views.osc2detail.visible = false
              else
                vb.views.osc2detail.visible = true
              end
              
              tmp.SAMPLE_TYPE_2 = wave
              OSC2.class = tmp.SAMPLE_TYPE_2
              redraw_sample()
            end
          },
        },
      },
    },
    -- ---------------------------------------- frequency osc 2
    vb:row{
      vb:text{text="Period", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:slider{
          id = "freq2slider",
          midi_mapping="MX:OSC2_period",
          width=140,
          min=1,
          max=64,
          value=tmp.SAMPLE_FREQUENCY_2,
          notifier = function(freq)
            tmp.SAMPLE_FREQUENCY_2 = freq
            OSC2.frequency = tmp.SAMPLE_FREQUENCY_2
            redraw_sample()
          end
        },
        vb:checkbox{
          id="osc2freqmod",
          width=16,
          height=16,
          value=false,
          notifier = function(bool)
            OSC2.modulate_freq=bool
            tmp.OSC2_FREQUENCY_MOD = bool
            redraw_sample()
          end
        },
      },
    },
    -- ----------------------------------------- amplitude OSC 2
    vb:row{
      vb:text{text="Amplitude", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:slider{
          id = "amp2slider",
          midi_mapping="MX:OSC2_amplitude",
          width=140,
          min=def.SAMPLE_MIN_AMPLITUDE,
          max=def.SAMPLE_MAX_AMPLITUDE,
          value=tmp.SAMPLE_AMPLITUDE_2,
          notifier = function(amplitude)
            tmp.SAMPLE_AMPLITUDE_2 = amplitude
            OSC2.amplitude = tmp.SAMPLE_AMPLITUDE_2
            redraw_sample()
          end
        },
        vb:checkbox{
          id="osc2ampmod",
          width=16,
          height=16,
          value=false,
          notifier = function(bool)
            OSC2.modulate_amp=bool
            tmp.OSC2_AMPLITUDE_MOD = bool
            redraw_sample()
          end
        },
      },
    },
    -- ------------------------------------------- phase OSC 2
    vb:row{
      vb:text{text="Phase", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:valuebox{
          id = "phase2slider",
          midi_mapping="MX:OSC2_phase",
          width=140,
          min=-def.SAMPLE_MAX_PHASE,
          max=def.SAMPLE_MAX_PHASE,
          value=tmp.SAMPLE_PHASE_2,
          steps={1, 10},
          notifier = function(phase)
            tmp.SAMPLE_PHASE_2 = phase
            OSC2.phase = tmp.SAMPLE_PHASE_2
            redraw_sample()
          end
        },
        vb:checkbox{
          id="osc2phasemod",
          width=16,
          height=16,
          value=false,
          notifier = function(bool)
            OSC2.modulate_phase=bool
            tmp.OSC2_PHASE_MOD = bool
            redraw_sample()
          end
        },
      },
    },
    ------------------------------------------------ Detail OSC2
    vb:row{
      id="osc2detail",
      vb:text{text="Detail", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:valuebox{
          id = "detail2slider",
          midi_mapping="MX:OSC2_detail",
          width=140,
          min=1,
          max=128,
          value=tmp.SAMPLE_DETAIL_2,
          steps={1, 10},
          notifier = function(detail)
            tmp.SAMPLE_DETAIL_2 = math.floor(detail)
            OSC2.detail = tmp.SAMPLE_DETAIL_2
            redraw_sample()
          end
        },
      },
    },
  }

--[[----------------------------------------------------------------
Create OSC 3 Settings                                              -
]]------------------------------------------------------------------
  local osc_3_gui = vb:column{
    id="osc3panel",
    style="group",
    margin=DEFAULT_MARGIN,
    vb:horizontal_aligner{
      mode="left",
      vb:text{
        text="Xochiquetzal",
        font="bold",
        style="strong",
        width=90,
      },
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:popup{
          id="osc3type",
          midi_mapping="MX:OSC3_selector",
          width=140,
          items=WAVES,
          value=tmp.SAMPLE_TYPE_3,
          notifier = function(wave)
            if wave < 5 then
              vb.views.osc3detail.visible = false
            else
              vb.views.osc3detail.visible = true
            end
            tmp.SAMPLE_TYPE_3 = wave
            OSC3.class = tmp.SAMPLE_TYPE_3
            redraw_sample()
          end
        },
      },
    },
    -- -------------------------------------------frequency OSC 3
    vb:row{
      vb:text{text="Period", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:slider{
          id = "freq3slider",
          midi_mapping="MX:OSC3_period",
          width=140,
          min=1,
          max=64,
          value=tmp.SAMPLE_FREQUENCY_3,
          notifier = function(freq)
            tmp.SAMPLE_FREQUENCY_3 = freq
            OSC3.frequency = tmp.SAMPLE_FREQUENCY_3
            redraw_sample()
          end
        },
        vb:checkbox{
          id="osc3freqmod",
          width=16,
          height=16,
          value=false,
          notifier = function(bool)
            OSC3.modulate_freq=bool
            tmp.OSC3_FREQUENCY_MOD = bool
            redraw_sample()
          end
        },
      },
    },
    -- -------------------------------------------- amplitude OSC 3
    vb:row{
      vb:text{text="Amplitude", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:slider{
          id = "amp3slider",
          midi_mapping="MX:OSC3_amplitude",
          width=140,
          min=def.SAMPLE_MIN_AMPLITUDE,
          max=def.SAMPLE_MAX_AMPLITUDE,
          value=tmp.SAMPLE_AMPLITUDE_3,
          notifier = function(amplitude)
            tmp.SAMPLE_AMPLITUDE_3 = amplitude
            OSC3.amplitude = tmp.SAMPLE_AMPLITUDE_3
            redraw_sample()
          end
        },
        vb:checkbox{
          id="osc3ampmod",
          width=16,
          height=16,
          value=false,
          notifier = function(bool)
            OSC3.modulate_amp=bool
            tmp.OSC3_AMPLITUDE_MOD = bool
            redraw_sample()
          end
        },
      },
    },
    -- -------------------------------------------- phase OSC 3
    vb:row{
      vb:text{text="Phase", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:valuebox{
          id = "phase3slider",
          midi_mapping="MX:OSC3_phase",
          width=140,
          min=-def.SAMPLE_MAX_PHASE,
          max=def.SAMPLE_MAX_PHASE,
          value=tmp.SAMPLE_PHASE_3,
          steps={1, 10},
          notifier = function(phase)
            tmp.SAMPLE_PHASE_3 = phase
            OSC3.phase = tmp.SAMPLE_PHASE_3
            redraw_sample()
          end
        },
        vb:checkbox{
          id="osc3phasemod",
          width=16,
          height=16,
          value=false,
          notifier = function(bool)
            OSC3.modulate_phase=bool
            tmp.OSC3_PHASE_MOD = bool
            redraw_sample()
          end
        },
      },
    },
    ------------------------------------------------ Detail OSC 3
    vb:row{
      id="osc3detail",
      vb:text{text="Detail", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:valuebox{
          id = "detail3slider",
          midi_mapping="MX:OSC3_detail",
          width=140,
          min=1,
          max=128,
          value=tmp.SAMPLE_DETAIL_3,
          steps={1, 10},
          notifier = function(detail)
            tmp.SAMPLE_DETAIL_3 = math.floor(detail)
            OSC3.detail = tmp.SAMPLE_DETAIL_3
            redraw_sample()
          end
        },
      },
    },
  }
      
--[[-------------------------------------------------------------------------------
Noisy Controls                                                                    -
]]---------------------------------------------------------------------------------
  local noisy_gui = vb:column{
    style="group",
    margin=DEFAULT_MARGIN,
    width=258,
    vb:row{
      vb:horizontal_aligner{
        mode="center",
        vb:text{
          width=200,
          id="noisy_message",
          text="Sak Nik sleeps",
          font="bold",
          style="strong",
        },
      },
    },
    vb:row{
      vb:text{text="Amount", align="left", width=90},
      vb:horizontal_aligner{
        mode="right",
        width=160,
        spacing=5,
        vb:slider{
          width=140,
          id="noise",
          midi_mapping="MX:Noise_amount",
          min=0,
          max=def.SAMPLE_MAX_NOISE,
          value=tmp.SAMPLE_NOISE_AMOUNT,
          notifier=function(noise)
            tmp.SAMPLE_NOISE_AMOUNT = noise
            if noise < 1 then
              tmp.SAMPLE_NOISE_BOOL = false
              vb.views.noisy_message.text="Sak Nik sleeps"
            else
              tmp.SAMPLE_NOISE_BOOL = true
              vb.views.noisy_message.text="Sak Nik is awake"
            end
            redraw_sample()
          end
        },
      },
    },
  }

--[[------------------------------------------------------------------------------------
Sample Frames control                                                                  -
]]--------------------------------------------------------------------------------------
  local frames_gui = vb:column{
    style="group",
    margin=DEFAULT_MARGIN,
    vb:row{
      vb:text{text="Sample frames", align="left", width=80},
      vb:horizontal_aligner{
        width=170,
        mode="right",
        spacing=5,
        vb:popup{
          id="framepopup",
          midi_mapping="MX:SAMPLE_LEN_selector",
          width=65,
          items=BUFFERS,
          value=tmp.SAMPLE_POPUP,
          notifier = function(amount)
            if MAIN_SAMPLE then
              read_info_from_sample(MAIN_SAMPLE, MAIN_INSTRUMENT)
            end

            tmp.SAMPLE_POPUP = math.floor(amount)
            tmp.SAMPLE_FRAMES = tonumber(BUFFERS[tmp.SAMPLE_POPUP])
            vb.views.framebox.value = tmp.SAMPLE_FRAMES
            redraw_sample("new_patch")
          end
        },
        vb:valuebox{
          id="framebox",
          width=70,
          min=1,
          max=44100,
          value=tmp.SAMPLE_FRAMES,
          steps={1, 10},
          notifier = function(frames)
            tmp.SAMPLE_FRAMES = frames
            redraw_sample("new_patch")
          end
        }
      },
    },
  }

--[[--------------------------------------------------------------------------------------
Operation control                                                                        -
]]----------------------------------------------------------------------------------------
  local operator_gui = vb:column{
    style="group",
    margin=DEFAULT_MARGIN,
    width=262,
    vb:horizontal_aligner{
      mode="center",
      vb:row{
        spacing=3,
        vb:text{text="Op1->", align="left"},
        vb:popup{
          midi_mapping="MX:OP1_selector",
          id="op1",
          width=70,
          items=OPERATIONS,
          value=tmp.OP1,
          notifier=function(value)
            tmp.OP1=tonumber(value)
            --print("OP1 changed to " .. tostring(OPERATIONS[value]))
            redraw_sample()
          end
        },
        vb:text{text="Op2->", align="left"},
        vb:popup{
          id="op2",
          midi_mapping="MX:OP2_selector",
          width=70,
          items=OPERATIONS,
          value = tmp.OP2,
          notifier=function(value)
            tmp.OP2 = tonumber(value)
            --print("OP2 changed to " .. tostring(OPERATIONS[value]))
            redraw_sample()
          end
        },
        vb:text{text="Op3", align="left"},
      },
    },
  }

--[[--------------------------------------------------------------------------------------
GAUSSIAN CONTROL                                                                         -
]]----------------------------------------------------------------------------------------
  local gaussian_gui = vb:column{
    spacing=DEFAULT_SPACING,
    margin = DEFAULT_MARGIN,
    height = DEFAULT_HEIGHT,
    vb:row{
        vb:checkbox{
          id="gaussian_on_off",
          midi_mapping="MX:Gaussian_toggle",
          value=tmp.GAUSSIAN_ON_OFF,
          notifier=function(bool)
            tmp.GAUSSIAN_ON_OFF = bool
            redraw_sample()
          end
        },
        vb:text{
          style="strong",
          font="bold",
          text="Let the gods work together",
          width=130,
        },
      },
    vb:column{
      id="gods1",
      spacing=DEFAULT_SPACING,
      margin = DEFAULT_MARGIN,
      height = DEFAULT_HEIGHT,
      style="group",
      vb:vertical_aligner{
        mode="center",
        vb:row{
          vb:text{text="Huehuecóyotl rising", width=130},
          vb:slider{
            id="gauss_v1",
            midi_mapping="MX:G1_rising",
            --height=MINISLIDER_HEIGHT,
            width=120,
            min=def.GAUSSIAN_V_MIN,
            max=def.GAUSSIAN_V_MAX,
            value=tmp.GAUSSIAN_V1,
            notifier = function(value)
              tmp.GAUSSIAN_V1 = value
              RECALCULATE = false
              redraw_sample()
            end
          },
        },
        vb:row{
          vb:text{text="Ah-Xoc-Xin rising", width=130},
          vb:slider{
            id="gauss_v2",
            midi_mapping="MX:G2_rising",
            --height=MINISLIDER_HEIGHT,
            width=120,
            min=def.GAUSSIAN_V_MIN,
            max=def.GAUSSIAN_V_MAX,
            value=tmp.GAUSSIAN_V2,
            notifier = function(value)
              tmp.GAUSSIAN_V2 = value
              RECALCULATE = false
              redraw_sample()
            end
          },
        },
        vb:row{
          vb:text{text="Xochiquetzal rising", width=130},
          vb:slider{
            id="gauss_v3",
            midi_mapping="MX:G3_rising",
            --height=MINISLIDER_HEIGHT,
            width=120,
            min=def.GAUSSIAN_V_MIN,
            max=def.GAUSSIAN_V_MAX,
            value=tmp.GAUSSIAN_V3,
            notifier = function(value)
              tmp.GAUSSIAN_V3 = value
              RECALCULATE = false
              redraw_sample()
            end
          },
        },
        vb:row{
          vb:text{text="Sak Nik rising", width=130},
          vb:slider{
            id="gauss_v4",
            midi_mapping="MX:G4_rising",
            --height=MINISLIDER_HEIGHT,
            width=120,
            min=def.GAUSSIAN_V_MIN,
            max=def.GAUSSIAN_V_MAX,
            value=tmp.GAUSSIAN_V4,
            notifier = function(value)
              tmp.GAUSSIAN_V4 = value
              RECALCULATE = false
              redraw_sample()
            end
          },
        },
        vb:row{
          vb:text{text="Macuilxochitl rising", width=130},
          vb:slider{
            id="gauss_v5",
            midi_mapping="MX:G5_rising",
            --height=MINISLIDER_HEIGHT,
            width=120,
            min=def.GAUSSIAN_V_MIN,
            max=def.GAUSSIAN_V_MAX,
            value=tmp.GAUSSIAN_V5,
            notifier = function(value)
              tmp.GAUSSIAN_V5 = value
              RECALCULATE = false
              redraw_sample()
            end
          },
        },
      },
    },
    vb:column{
      id="gods2",
      spacing=DEFAULT_SPACING,
      margin = DEFAULT_MARGIN,
      height = DEFAULT_HEIGHT,
      style="group",
      vb:row{
        vb:text{text="Huehuecóyotl dancing", width=130},
        vb:slider{
          id="gauss_n1",
          midi_mapping="MX:G1_dancing",
          --height=MINISLIDER_HEIGHT,
          width=120,
          min=def.GAUSSIAN_N_MIN,
          max=def.GAUSSIAN_N_MAX,
          value=tmp.GAUSSIAN_N1,
          notifier = function(value)
            tmp.GAUSSIAN_N1 = value
            RECALCULATE = false
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Ah-Xoc-Xin dancing", width=130},
        vb:slider{
          id="gauss_n2",
          midi_mapping="MX:G2_dancing",
          --height=MINISLIDER_HEIGHT,
          width=120,
          min=def.GAUSSIAN_N_MIN,
          max=def.GAUSSIAN_N_MAX,
          value=tmp.GAUSSIAN_N2,
          notifier = function(value)
            tmp.GAUSSIAN_N2 = value
            RECALCULATE = false
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Xochiquetzal dancing", width=130},
        vb:slider{
          id="gauss_n3",
          midi_mapping="MX:G3_dancing",
          --height=MINISLIDER_HEIGHT,
          width=120,
          min=def.GAUSSIAN_N_MIN,
          max=def.GAUSSIAN_N_MAX,
          value=tmp.GAUSSIAN_N3,
          notifier = function(value)
            tmp.GAUSSIAN_N3 = value
            RECALCULATE = false
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Sak Nik dancing", width=130},
        vb:slider{
          id="gauss_n4",
          midi_mapping="MX:G4_dancing",
          --height=MINISLIDER_HEIGHT,
          width=120,
          min=def.GAUSSIAN_N_MIN,
          max=def.GAUSSIAN_N_MAX,
          value=tmp.GAUSSIAN_N4,
          notifier = function(value)
            tmp.GAUSSIAN_N4 = value
            RECALCULATE = false
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Macuilxochitl dancing", width=130},
        vb:slider{
          id="gauss_n5",
          midi_mapping="MX:G5_dancing",
          --height=MINISLIDER_HEIGHT,
          width=120,
          min=def.GAUSSIAN_N_MIN,
          max=def.GAUSSIAN_N_MAX,
          value=tmp.GAUSSIAN_N5,
          notifier = function(value)
            tmp.GAUSSIAN_N5 = value
            RECALCULATE = false
            redraw_sample()
          end
        },
      },
    },
    vb:column{
      id="gods3",
      spacing=DEFAULT_SPACING,
      margin = DEFAULT_MARGIN,
      height = DEFAULT_HEIGHT,
      style="group",
      vb:row{
        vb:text{text="Huehuecóyotl expands", width=130},
        vb:slider{
          id="gauss_s1",
          midi_mapping="MX:G1_expands",
          --height=MINISLIDER_HEIGHT,
          width=120,
          min=def.GAUSSIAN_S_MIN,
          max=def.GAUSSIAN_S_MAX,
          value=tmp.GAUSSIAN_S1,
          notifier = function(value)
            tmp.GAUSSIAN_S1 = value
            RECALCULATE = false
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Ah-Xoc-Xin expands", width=130},
        vb:slider{
          id="gauss_s2",
          midi_mapping="MX:G2_expands",
          --height=MINISLIDER_HEIGHT,
          width=120,
          min=def.GAUSSIAN_S_MIN,
          max=def.GAUSSIAN_S_MAX,
          value=tmp.GAUSSIAN_S2,
          notifier = function(value)
            tmp.GAUSSIAN_S2 = value
            RECALCULATE = false
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Xochiquetzal expands", width=130},
        vb:slider{
          id="gauss_s3",
          midi_mapping="MX:G3_expands",
          --height=MINISLIDER_HEIGHT,
          width=120,
          min=def.GAUSSIAN_S_MIN,
          max=def.GAUSSIAN_S_MAX,
          value=tmp.GAUSSIAN_S3,
          notifier = function(value)
            tmp.GAUSSIAN_S3 = value
            RECALCULATE = false
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Sak Nik expands", width=130},
        vb:slider{
          id="gauss_s4",
          midi_mapping="MX:G4_expands",
          --height=MINISLIDER_HEIGHT,
          width=120,
          min=def.GAUSSIAN_S_MIN,
          max=def.GAUSSIAN_S_MAX,
          value=tmp.GAUSSIAN_S4,
          notifier = function(value)
            tmp.GAUSSIAN_S4 = value
            RECALCULATE = false
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Macuilxochitl expands", width=130},
        vb:slider{
          id="gauss_s5",
          midi_mapping="MX:G5_expands",
          --height=MINISLIDER_HEIGHT,
          width=120,
          min=def.GAUSSIAN_S_MIN,
          max=def.GAUSSIAN_S_MAX,
          value=tmp.GAUSSIAN_S5,
          notifier = function(value)
            tmp.GAUSSIAN_S5 = value
            RECALCULATE = false
            redraw_sample()
          end
        },
      },
    },
  }

  --[[------------------------------------------------------------------------
  Patches page                                                               -
  ]]--------------------------------------------------------------------------
  local patches_gui = vb:column{
    spacing=DEFAULT_SPACING,
    margin = DEFAULT_MARGIN,
    height = DEFAULT_HEIGHT,
    width=264,
    style="group",
    uniform=true,
    vb:space{height=50},
    vb:horizontal_aligner{
      mode="center",
      vb:button{
        text="Load patch",
        width=200,
        height=32,
        notifier=function()
          local filename = renoise.app():prompt_for_filename_to_read({"*.mxp"}, "Load Macuilxochitl patch")
          if #filename ~= 0 then
            load_patch_file(filename)
            redraw_sample("new_patch")
            vb.views.switcher.value = 1
            vb.views.page1.visible = true
            vb.views.page2.visible = false
            vb.views.page3.visible = false
            vb.views.page4.visible = false
          end
        end
      },
    },
    vb:space{height=30},
    vb:horizontal_aligner{
      mode="center",
      vb:button{
        width=200,
        height=32,
        text="Save patch",
        notifier=function()
          local filename = renoise.app():prompt_for_filename_to_write("mxp", "Save Macuilxochitl patch")
          if #filename ~= 0 then
            save_patch_file(filename)
            vb.views.switcher.value = 1
            vb.views.page1.visible = true
            vb.views.page2.visible = false
            vb.views.page3.visible = false
            vb.views.page4.visible = false
          end
        end
      },
    },
    vb:space{height=30},
    vb:horizontal_aligner{
      mode="center",
      vb:button{
        color={180, 0, 0},
        width=200,
        height=32,
        text="Reset patch",
        notifier=function()
          local answer = renoise.app():show_prompt("Reset patch question", "Really RESET Patch ?", {"Yes", "No"})
          if answer == "Yes" then
            load_patch_file()
            redraw_sample("new_patch")
            vb.views.switcher.value = 1
            vb.views.page1.visible = true
            vb.views.page2.visible = false
            vb.views.page3.visible = false
            vb.views.page4.visible = false
          end
        end
      },
    },
    vb:space{height=100},
  }

  --[[------------------------------------------------------------------------
  Some little info                                                           -
  ]]--------------------------------------------------------------------------
  local info_page = vb:column{
    spacing=DEFAULT_SPACING,
    margin = DEFAULT_MARGIN,
    height = DEFAULT_HEIGHT,
    style="group",
    width=262,
    vb:horizontal_aligner{
      mode="center",
      vb:bitmap{
        mode="transparent",
        bitmap="gfx/info.bmp",
      },
    },
    vb:horizontal_aligner{
      mode="center",
      vb:vertical_aligner{
        mode="top",
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            font="bold",
            style="strong",
            align="center",
            text="Macuilxochitl v" .. VERSION,
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            --font="bold",
            --style="strong",
            align="center",
            text="created by",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            --font="bold",
            --style="strong",
            align="center",
            text=AUTHOR,
          },
        },
        vb:space{height=10},
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            font="italic",
            align="center",
            text="Is the Mesoamerican god of music and dance",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            align="center",
            font="italic",
            text="and an experimental extension for Renoise.",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            style="strong",
            font="italic",
            align="center",
            text="Please be careful and protect your hearing !",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            font="italic",
            align="center",
            text="Some of the gods are still quite unstable",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            font="italic",
            align="center",
            text="because they are trapped in an unstable code.",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            font="italic",
            align="center",
            text="If you like the extension, use it.",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            font="italic",
            align="center",
            text="Thanks",
          },
        },
        vb:space{height=30},
      },
    },
  }

  --[[------------------------------------------------------------------------
  Main gui                                                                   -
  ]]--------------------------------------------------------------------------
  local make_gui = vb:column{
    spacing=DEFAULT_SPACING,
    margin=DEFAULT_MARGIN,
    height=DEFAULT_HEIGHT,
    vb:horizontal_aligner{
      mode="center",
      id="switch",
      vb:switch{
        id="switcher",
        width=260,
        height=24,
        items={"Oscils", "Gaussians", "Patches", "About"},
        value = 1,
        notifier = function(page)
          if page == 1 then
            vb.views.page1.visible = true
            vb.views.page2.visible = false
            vb.views.page3.visible = false
            vb.views.page4.visible = false
          elseif page == 2 then
            vb.views.page1.visible = false
            vb.views.page2.visible = true
            vb.views.page3.visible = false
            vb.views.page4.visible = false
          elseif page == 3 then
            vb.views.page1.visible = false
            vb.views.page2.visible = false
            vb.views.page3.visible = true
            vb.views.page4.visible = false
          elseif page == 4 then
            vb.views.page1.visible = false
            vb.views.page2.visible = false
            vb.views.page3.visible = false
            vb.views.page4.visible = true
          end
        end
      },
    },
    vb:column{
      spacing=DEFAULT_SPACING,
      margin = DEFAULT_MARGIN,
      height = DEFAULT_HEIGHT,
      id="page1",
      osc_1_gui,
      osc_2_gui,
      osc_3_gui,
      noisy_gui,
      frames_gui,
      operator_gui,
    },
    vb:column{
      id="page2",
      gaussian_gui,
    },
    vb:column{
      id="page4",
      info_page,
    },
    vb:column{
      id="page3",
      patches_gui,
    },
    --vb:bitmap{
    --  id="logo_bitmap",
    --  mode="transparent",
    --  bitmap="gfx/info2.bmp"
    --},
  }

  vb.views.switch.visible=true
  vb.views.page1.visible=true
  vb.views.page2.visible=false
  vb.views.page3.visible=false
  vb.views.page4.visible=false
  --vb.views.logo_bitmap.visible=true

  -- in default is simple wave set
  vb.views.osc1detail.visible = false
  vb.views.osc2detail.visible = false
  vb.views.osc3detail.visible = false

  -- sample buffer values
  vb.views.framepopup.value = tmp.SAMPLE_POPUP

  Notifiers:add(rnt.app_idle_observable, app_idle)

  redraw_sample("new_start")
  
  dialog = renoise.app():show_custom_dialog(
    "Macuilxochitl",
    make_gui)

end